/*
 *  CSDecoder.java
 *  Copyright (C) 2000  P. Taylor Goetz <tgoetz@fast.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package catscratch;

import java.util.StringTokenizer;

/** Utility class for decoding strings produced by certain animal-shaped
 * barcode scanning devices. Based on a perl script by Kevin Fowlks.
 * @author P. Taylor Goetz
 * @version 1.0
 */
public class CSDecoder {

    /** Creates new CSDecoder */
    private CSDecoder() {
    }

    /** Decodes the string produced by the scanning device and returns a CatScan
     * object containing the unencoded data.
     * @param src String containing the output of the device.
     */
    public static CatScan decode(String src){
        //
        String temp = null;
        String[] data = CSDecoder.explode(src);
        // Sanity Check: expecting 3 fields: id, barcode type, barcode
        if(data.length == 3){
            for(int i=0;i<data.length;i++){
                temp = data[i];
                // flip uppper case <--> lower case
                temp = CSDecoder.flipCase(temp);
                // replace minuses w/slashes
                temp = temp.replace('-', '/');
                // base64 decode
                temp = CSDecoder.xor67(Base64Decoder.decode(temp));
                data[i] = temp;
            }
            return new CatScan(data[0], data[1], data[2]);
        }
        else{
            // bad format
            return null;
        }
    }

    /** Takes each byte XOR 67.
     * @param b byte array source to be XOR'ed by 67
     * @return String
     */
    protected static String xor67(byte[] b){
        // XOR 67 each character
        for(int i=0;i<b.length;i++){
            b[i] = (byte)((int)b[i] ^ 67);
        }
        return new String(b);
    }

    /** Parses a period('.')-delimited String into a String array.
     */
    protected static String[] explode(String src){
        StringTokenizer parser = new StringTokenizer(src, ".");
        int tokenCount = parser.countTokens();
        String[] retVal = new String[tokenCount];
        int i = 0;
        while(parser.hasMoreElements()){
            retVal[i] = (String)parser.nextElement();
            i++;
        }
        return retVal;
    }

    /** Toggles the case of each character in a String.<br>
     * <br>
     * <PRE>For example: AbC --> aBc</PRE>
     */
    protected static String flipCase(String src){
        char[] chars = src.toCharArray();
        for(int i=0;i<chars.length;i++){
            if(Character.isUpperCase(chars[i])){
                chars[i] = Character.toLowerCase(chars[i]);
            }
            else if(Character.isLowerCase(chars[i])){
                chars[i] = Character.toUpperCase(chars[i]);
            }
            // otherwise, some other char, leave it alone.
        }
        return new String(chars);
    }

}